from .application_properties import ApplicationProperties
from .server_properties import ServerProperties
from .applications import AutoBoot, AutoBootConfig

VERSION  = "0.8.0"

__all__ = ["AutoBoot", "AutoBootConfig", "ApplicationProperties", "ServerProperties"]
